@@ -29,41 +29,40 @@ struct SDMABUFModifier {
 
 class CHyprlock {
   public:
-    CHyprlock(const std::string& wlDisplay, const bool immediate, const bool immediateRender);
+    CHyprlock(const std::string& wlDisplay, const bool immediateRender, const int gracePeriod);
     ~CHyprlock();
 
-    void                             run();
+    void                       run();
 
-    void                             unlock();
-    bool                             isUnlocked();
+    void                       unlock();
+    bool                       isUnlocked();
 
-    std::shared_ptr<CTimer>          addTimer(const std::chrono::system_clock::duration& timeout, std::function<void(std::shared_ptr<CTimer> self, void* data)> cb_, void* data,
-                                              bool force = false);
+    ASP<CTimer>                addTimer(const std::chrono::system_clock::duration& timeout, std::function<void(ASP<CTimer> self, void* data)> cb_, void* data, bool force = false);
 
-    void                             enqueueForceUpdateTimers();
+    void                       enqueueForceUpdateTimers();
 
-    void                             onLockLocked();
-    void                             onLockFinished();
+    void                       onLockLocked();
+    void                       onLockFinished();
 
-    bool                             acquireSessionLock();
-    void                             releaseSessionLock();
+    bool                       acquireSessionLock();
+    void                       releaseSessionLock();
 
-    std::string                      spawnSync(const std::string& cmd);
+    void                       onKey(uint32_t key, bool down);
+    void                       onClick(uint32_t button, bool down, const Vector2D& pos);
+    void                       onHover(const Vector2D& pos);
+    void                       startKeyRepeat(xkb_keysym_t sym);
+    void                       repeatKey(xkb_keysym_t sym);
+    void                       handleKeySym(xkb_keysym_t sym, bool compose);
+    void                       onPasswordCheckTimer();
+    void                       clearPasswordBuffer();
+    bool                       passwordCheckWaiting();
+    std::optional<std::string> passwordLastFailReason();
 
-    void                             onKey(uint32_t key, bool down);
-    void                             startKeyRepeat(xkb_keysym_t sym);
-    void                             repeatKey(xkb_keysym_t sym);
-    void                             handleKeySym(xkb_keysym_t sym, bool compose);
-    void                             onPasswordCheckTimer();
-    void                             clearPasswordBuffer();
-    bool                             passwordCheckWaiting();
-    std::optional<std::string>       passwordLastFailReason();
+    void                       renderOutput(const std::string& stringPort);
+    void                       renderAllOutputs();
 
-    void                             renderOutput(const std::string& stringPort);
-    void                             renderAllOutputs();
-
-    size_t                           getPasswordBufferLen();
-    size_t                           getPasswordBufferDisplayLen();
+    size_t                     getPasswordBufferLen();
+    size_t                     getPasswordBufferDisplayLen();
 
     SP<CCExtSessionLockManagerV1>    getSessionLockMgr();
     SP<CCExtSessionLockV1>           getSessionLock();
@@ -95,11 +94,14 @@ class CHyprlock {
     //
     std::chrono::system_clock::time_point m_tGraceEnds;
     Vector2D                              m_vLastEnterCoords = {};
+    WP<COutput>                           m_focusedOutput;
 
-    std::shared_ptr<CTimer>               m_pKeyRepeatTimer = nullptr;
+    Vector2D                              m_vMouseLocation = {};
+
+    ASP<CTimer>                           m_pKeyRepeatTimer = nullptr;
 
     std::vector<SP<COutput>>              m_vOutputs;
-    std::vector<std::shared_ptr<CTimer>>  getTimers();
+    std::vector<ASP<CTimer>>              getTimers();
 
     struct {
         SP<CCZwpLinuxDmabufV1>         linuxDmabuf         = nullptr;
@@ -116,6 +118,9 @@ class CHyprlock {
     } dma;
     gbm_device* createGBMDevice(drmDevice* dev);
 
+    void        addDmabufListener();
+    void        removeDmabufListener();
+
   private:
     struct {
         wl_display*                      display     = nullptr;
@@ -128,8 +133,6 @@ class CHyprlock {
         SP<CCWlShm>                      shm         = nullptr;
     } m_sWaylandState;
 
-    void addDmabufListener();
-
     struct {
         SP<CCExtSessionLockV1> lock = nullptr;
     } m_sLockState;
@@ -155,11 +158,9 @@ class CHyprlock {
         bool                    timerEvent = false;
     } m_sLoopState;
 
-    bool                                 m_bUnlockedCalled = false;
+    std::vector<ASP<CTimer>> m_vTimers;
 
-    std::vector<std::shared_ptr<CTimer>> m_vTimers;
-
-    std::vector<uint32_t>                m_vPressedKeys;
+    std::vector<uint32_t>    m_vPressedKeys;
 };
 
 inline UP<CHyprlock> g_pHyprlock;
