@@ -2,6 +2,7 @@
 
 #include <hyprutils/memory/WeakPtr.hpp>
 #include <hyprutils/memory/UniquePtr.hpp>
+#include <hyprutils/memory/Atomic.hpp>
 #include <hyprgraphics/color/Color.hpp>
 
 using namespace Hyprutils::Memory;
@@ -10,5 +11,8 @@ using namespace Hyprgraphics;
 #define WP CWeakPointer
 #define UP CUniquePointer
 
+#define ASP CAtomicSharedPointer
+#define AWP CAtomicWeakPointer
+
 typedef int64_t    OUTPUTID;
 constexpr OUTPUTID OUTPUT_INVALID = -1;
