@@ -1,7 +1,7 @@
 #pragma once
 
-#include "../../helpers/Math.hpp"
 #include "../../defines.hpp"
+#include "../../helpers/Math.hpp"
 #include <string>
 #include <unordered_map>
 #include <any>
@@ -24,6 +24,13 @@ class IWidget {
     static int      roundingForBox(const CBox& box, int roundingConfig);
     static int      roundingForBorderBox(const CBox& borderBox, int roundingConfig, int thickness);
 
+    virtual CBox    getBoundingBoxWl() const {
+        return CBox();
+    };
+    virtual void onClick(uint32_t button, bool down, const Vector2D& pos) {}
+    virtual void onHover(const Vector2D& pos) {}
+    bool         containsPoint(const Vector2D& pos) const;
+
     struct SFormatResult {
         std::string formatted;
         float       updateEveryMs    = 0; // 0 means don't (static)
@@ -33,4 +40,10 @@ class IWidget {
     };
 
     static SFormatResult formatString(std::string in);
+
+    void                 setHover(bool hover);
+    bool                 isHovered() const;
+
+  private:
+    bool hovered = false;
 };
