@@ -1,5 +1,6 @@
 #pragma once
 
+#include "../../defines.hpp"
 #include "IWidget.hpp"
 #include "../../helpers/Color.hpp"
 #include "../../helpers/Math.hpp"
@@ -20,10 +21,13 @@ class CImage : public IWidget {
     CImage() = default;
     ~CImage();
 
-    void         registerSelf(const SP<CImage>& self);
+    void         registerSelf(const ASP<CImage>& self);
 
     virtual void configure(const std::unordered_map<std::string, std::any>& props, const SP<COutput>& pOutput);
     virtual bool draw(const SRenderData& data);
+    virtual CBox getBoundingBoxWl() const;
+    virtual void onClick(uint32_t button, bool down, const Vector2D& pos);
+    virtual void onHover(const Vector2D& pos);
 
     void         reset();
 
@@ -32,7 +36,7 @@ class CImage : public IWidget {
     void         plantTimer();
 
   private:
-    WP<CImage>                              m_self;
+    AWP<CImage>                             m_self;
 
     CFramebuffer                            imageFB;
 
@@ -42,6 +46,7 @@ class CImage : public IWidget {
     double                                  angle;
     CGradientValueData                      color;
     Vector2D                                pos;
+    Vector2D                                configPos;
 
     std::string                             halign, valign, path;
 
@@ -49,8 +54,10 @@ class CImage : public IWidget {
 
     int                                     reloadTime;
     std::string                             reloadCommand;
+    std::string                             onclickCommand;
+
     std::filesystem::file_time_type         modificationTime;
-    std::shared_ptr<CTimer>                 imageTimer;
+    ASP<CTimer>                             imageTimer;
     CAsyncResourceGatherer::SPreloadRequest request;
 
     Vector2D                                viewport;
@@ -58,6 +65,6 @@ class CImage : public IWidget {
 
     std::string                             resourceID;
     std::string                             pendingResourceID; // if reloading image
-    SPreloadedAsset*                        asset = nullptr;
+    ASP<SPreloadedAsset>                    asset = nullptr;
     CShadowable                             shadow;
 };
