@@ -19,7 +19,7 @@ CPasswordInputField::~CPasswordInputField() {
     reset();
 }
 
-void CPasswordInputField::registerSelf(const SP<CPasswordInputField>& self) {
+void CPasswordInputField::registerSelf(const ASP<CPasswordInputField>& self) {
     m_self = self;
 }
 
@@ -29,7 +29,7 @@ void CPasswordInputField::configure(const std::unordered_map<std::string, std::a
     outputStringPort = pOutput->stringPort;
     viewport         = pOutput->getViewport();
 
-    shadow.configure(m_self.lock(), props, viewport);
+    shadow.configure(m_self, props, viewport);
 
     try {
         pos                      = CLayoutValueData::fromAnyPv(props.at("position"))->getAbsolute(viewport);
@@ -95,7 +95,7 @@ void CPasswordInputField::configure(const std::unordered_map<std::string, std::a
         request.props["color"]       = colorConfig.font;
         request.props["font_size"]   = (int)(std::nearbyint(configSize.y * dots.size * 0.5f) * 2.f);
 
-        g_pRenderer->asyncResourceGatherer->requestAsyncAssetPreload(request);
+        g_pAsyncResourceGatherer->requestAsyncAssetPreload(request);
     }
 
     // request the inital placeholder asset
@@ -112,14 +112,14 @@ void CPasswordInputField::reset() {
         return;
 
     if (placeholder.asset)
-        g_pRenderer->asyncResourceGatherer->unloadAsset(placeholder.asset);
+        g_pAsyncResourceGatherer->unloadAsset(placeholder.asset);
 
     placeholder.asset = nullptr;
     placeholder.resourceID.clear();
     placeholder.currentText.clear();
 }
 
-static void fadeOutCallback(WP<CPasswordInputField> ref) {
+static void fadeOutCallback(AWP<CPasswordInputField> ref) {
     if (const auto PP = ref.lock(); PP)
         PP->onFadeOutTimer();
 }
@@ -242,7 +242,7 @@ bool CPasswordInputField::draw(const SRenderData& data) {
 
         if (!dots.textFormat.empty()) {
             if (!dots.textAsset)
-                dots.textAsset = g_pRenderer->asyncResourceGatherer->getAssetByID(dots.textResourceID);
+                dots.textAsset = g_pAsyncResourceGatherer->getAssetByID(dots.textResourceID);
 
             if (!dots.textAsset)
                 forceReload = true;
@@ -302,10 +302,10 @@ bool CPasswordInputField::draw(const SRenderData& data) {
     }
 
     if (passwordLength == 0 && !checkWaiting && !placeholder.resourceID.empty()) {
-        SPreloadedAsset* currAsset = nullptr;
+        ASP<SPreloadedAsset> currAsset = nullptr;
 
         if (!placeholder.asset)
-            placeholder.asset = g_pRenderer->asyncResourceGatherer->getAssetByID(placeholder.resourceID);
+            placeholder.asset = g_pAsyncResourceGatherer->getAssetByID(placeholder.resourceID);
 
         currAsset = placeholder.asset;
 
@@ -330,7 +330,7 @@ void CPasswordInputField::updatePlaceholder() {
     if (passwordLength != 0) {
         if (placeholder.asset && /* keep prompt asset cause it is likely to be used again */ displayFail) {
             std::erase(placeholder.registeredResourceIDs, placeholder.resourceID);
-            g_pRenderer->asyncResourceGatherer->unloadAsset(placeholder.asset);
+            g_pAsyncResourceGatherer->unloadAsset(placeholder.asset);
             placeholder.asset      = nullptr;
             placeholder.resourceID = "";
             redrawShadow           = true;
@@ -351,8 +351,7 @@ void CPasswordInputField::updatePlaceholder() {
     if (!ALLOWCOLORSWAP && newText == placeholder.currentText)
         return;
 
-    const auto NEWRESOURCEID =
-        std::format("placeholder:{}{}{}{}{}{}", placeholder.currentText, (uintptr_t)this, colorState.font.r, colorState.font.g, colorState.font.b, colorState.font.a);
+    const auto NEWRESOURCEID = std::format("placeholder:{}{}{}{}{}{}", newText, (uintptr_t)this, colorState.font.r, colorState.font.g, colorState.font.b, colorState.font.a);
 
     if (placeholder.resourceID == NEWRESOURCEID)
         return;
@@ -380,7 +379,7 @@ void CPasswordInputField::updatePlaceholder() {
         if (const auto SELF = REF.lock(); SELF)
             g_pHyprlock->renderOutput(SELF->outputStringPort);
     };
-    g_pRenderer->asyncResourceGatherer->requestAsyncAssetPreload(request);
+    g_pAsyncResourceGatherer->requestAsyncAssetPreload(request);
 }
 
 void CPasswordInputField::updateWidth() {
@@ -473,3 +472,14 @@ void CPasswordInputField::updateColors() {
 
     colorState.font = fontTarget;
 }
+
+CBox CPasswordInputField::getBoundingBoxWl() const {
+    return {
+        Vector2D{pos.x, viewport.y - pos.y - size->value().y},
+        size->value(),
+    };
+}
+
+void CPasswordInputField::onHover(const Vector2D& pos) {
+    g_pSeatManager->m_pCursorShape->setShape(WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_TEXT);
+}
